// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package middleware

import (
	"net/http"

	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/core"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/errno"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/token"
	"github.com/gin-gonic/gin"
)

var ErrTokenInvalid = &errno.Errno{
	HTTP:    http.StatusUnauthorized,
	Code:    errno.ErrorCode(errno.AuthFailureCode, "TokenInvalid"),
	Message: "invalid token",
}

const XUserName = "X-Username"

// Authn authenticates a user's access by validating their token.
func Authn() gin.HandlerFunc {
	return func(ctx *gin.Context) {
		tk, err := token.ParseRequest(ctx)
		if err != nil || tk == nil {
			core.WriteResponse(ctx, ErrTokenInvalid, nil)
			ctx.Abort()
		}

		// TODO: check if the key is on logout blacklist.

		ctx.Header(XUserName, tk.Identity)
		ctx.Next()
	}
}
